@@ -18,6 +18,7 @@
 #include <termios.h>
 #include <signal.h>
 #include <fcntl.h>
+#include <stdio.h>
 #ifdef WITH_PROFILER
 #include <gperftools/profiler.h>
 #endif
@@ -98,7 +99,7 @@ start_profiler(PyObject UNUSED *self, PyObject *args) {
 }
 
 static PyObject*
-stop_profiler(PyObject UNUSED *self) {
+stop_profiler(PyObject UNUSED *self, PyObject *args UNUSED) {
     ProfilerStop();
     Py_RETURN_NONE;
 }
@@ -123,7 +124,9 @@ open_tty(PyObject *self UNUSED, PyObject *args) {
     if (!PyArg_ParseTuple(args, "|p", &read_with_timeout)) return NULL;
     int flags = O_RDWR | O_CLOEXEC | O_NOCTTY;
     if (!read_with_timeout) flags |= O_NONBLOCK;
-    int fd = open("/dev/tty", flags);
+    static char ctty[L_ctermid+1];
+    int fd = open(ctermid(ctty), flags);
+    if (fd == -1) { PyErr_SetFromErrno(PyExc_OSError); return NULL; }
     struct termios *termios_p = calloc(1, sizeof(struct termios));
     if (!termios_p) return PyErr_NoMemory();
     if (tcgetattr(fd, termios_p) != 0) { free(termios_p); PyErr_SetFromErrno(PyExc_OSError); return NULL; }
@@ -215,6 +218,7 @@ extern bool init_png_reader(PyObject *module);
 #ifdef __APPLE__
 extern int init_CoreText(PyObject *);
 extern bool init_cocoa(PyObject *module);
+extern bool init_macos_process_info(PyObject *module);
 #else
 extern bool init_freetype_library(PyObject*);
 #endif
@@ -249,6 +253,7 @@ PyInit_fast_data_types(void) {
         if (!init_kittens(m)) return NULL;
         if (!init_png_reader(m)) return NULL;
 #ifdef __APPLE__
+        if (!init_macos_process_info(m)) return NULL;
         if (!init_CoreText(m)) return NULL;
         if (!init_cocoa(m)) return NULL;
 #else
